;; -*- no-byte-compile: t; lexical-binding: nil -*-
(define-package "emms-state" "0.2.1"
  "Display EMMS track description and playing time in the mode line."
  '((emms "0"))
  :url "https://github.com/alezost/emms-state.el"
  :commit "18128405c0a3659f493fceb3ea22e0a8a39125be"
  :revdesc "18128405c0a3"
  :keywords '("emms")
  :authors '(("Alex Kost" . "alezost@gmail.com"))
  :maintainers '(("Alex Kost" . "alezost@gmail.com")))
